/*
 * C utility library
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "library.h"
#include "log.h"
#include "config.h"

#ifndef INADDR_NONE
#define INADDR_NONE ((unsigned long int)0xffffffff)
#endif

/* Fill in the boring fields and do all the boring checking stuff */
static void
log_boring(errmsg_t *msg)
{
	char	*site;
	char	*port;
	char	*url;

	msg->version = 0;

	if (!msg->pid)
		msg->pid = getpid();

	if (!*msg->site) {
		site = getenv(ENV_HOSTNAME);
		if (!site) site = NO_SITE;
		strncpy(msg->site, site, (LOG_SITELEN - 1));
	}
	msg->site[(LOG_SITELEN - 1)] = 0;

	if (!msg->port) {
		port = getenv("SERVER_PORT");
		if (!port) msg->port = -1;
		else msg->port = atol(port);
	}

	if (!*msg->referer) {
		url = getenv("HTTP_REFERER");
		if (!url)
			url = NO_REFERER;
		strncpy(msg->referer, url, (LOG_REFERERLEN - 1));
	}
	msg->referer[(LOG_REFERERLEN - 1)] = 0;

	if (!*msg->url) {
		url = getenv("REQUEST_URI");
		if (!url)
			url = getenv("SCRIPT_NAME");
		if (!url)
			url = NO_URL;
		strncpy(msg->url, url, (LOG_URLLEN - 1));
	}
	msg->url[(LOG_URLLEN - 1)] = 0;

	strncpy(msg->program, config_name(), (LOG_PROGRAMLEN - 1));
	msg->program[(LOG_PROGRAMLEN - 1)] = 0;

	msg->facility[(LOG_FACILITYLEN - 1)] = 0;

	msg->version = htonl(msg->version);
	msg->pid = htonl(msg->pid);
	msg->port = htonl(msg->port);
	msg->priority = htonl(msg->priority);
}

/* Don't static this, we might want to call it ourselves */
/* This has to perror() as it might not be able to log */
int
log_msg(errmsg_t *msg, char *host, int port)
{
	struct sockaddr_in	 saddr;
	struct protoent		*proto;
	struct hostent		*hostp;
	unsigned int		 ipaddr;
	int					 sock;

	log_boring(msg);

	proto = getprotobyname("udp");
	sock = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
	if (sock < 0) {
		perror("socket");	/* To stderr */
		return -1;
	}

	ipaddr = inet_addr(host);
	if (ipaddr == INADDR_NONE) {
		hostp = gethostbyname(host);
		if (!hostp) {
			perror("gethostbyname");
			return -1;
		}
		memcpy(&ipaddr, hostp->h_addr, sizeof(ipaddr));
	}

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);
	saddr.sin_addr.s_addr = ipaddr;

	if ((sendto(sock, ((void *)(msg)), sizeof(errmsg_t), 0,
			((struct sockaddr *)(&saddr)), sizeof(saddr))) < 0)
	{
		perror("sendto");
		return -1;
	}
	return 0;
}

/* Complete the interesting fields */
void
log(char *facility, int priority, char *format, ...)
{
	errmsg_t	 msg;
	va_list		 args;

	memset(&msg, 0, sizeof(msg));

	if (!facility) facility = NO_FACILITY;
	strncpy(msg.facility, facility, (LOG_FACILITYLEN - 1));

	msg.priority = priority;

	va_start(args, format);
	vsnprintf(msg.message, LOG_MESSAGELEN - 1, format, args);
	msg.message[(LOG_MESSAGELEN - 1)] = 0;
	va_end(args);

	log_msg(&msg, LOG_HOST, LOG_PORT);
}
